@@ -39,7 +39,8 @@ _AUTH_ALLOWED_PARAMETERS_MAPPING = {
     'sslcrl': 'PGSSLCRL',
     'sslcrldir': 'PGSSLCRLDIR',
     'gssencmode': 'PGGSSENCMODE',
-    'channel_binding': 'PGCHANNELBINDING'
+    'channel_binding': 'PGCHANNELBINDING',
+    'sslnegotiation': 'PGSSLNEGOTIATION'
 }
 NO_VALUE_MSG = '#FIXME'
 
@@ -53,13 +54,15 @@ def get_address() -> Tuple[str, str]:
     :returns: tuple consisting of the hostname returned by :func:`~socket.gethostname`
         and the first element in the sorted list of the addresses returned by :func:`~socket.getaddrinfo`.
         Sorting guarantees it will prefer IPv4.
-        If an exception occured, hostname and ip values are equal to :data:`~patroni.config_generator.NO_VALUE_MSG`.
+        If an exception occurred, hostname and ip values are equal to :data:`~patroni.config_generator.NO_VALUE_MSG`.
     """
     hostname = None
     try:
         hostname = socket.gethostname()
-        return hostname, sorted(socket.getaddrinfo(hostname, 0, socket.AF_UNSPEC, socket.SOCK_STREAM, 0),
-                                key=lambda x: x[0])[0][4][0]
+        # Filter out unexpected results when python is compiled with --disable-ipv6 and running on IPv6 system.
+        addrs = [(a[0], a[4][0]) for a in socket.getaddrinfo(hostname, 0, socket.AF_UNSPEC, socket.SOCK_STREAM, 0)
+                 if isinstance(a[4][0], str)]
+        return hostname, sorted(addrs, key=lambda x: x[0])[0][1]
     except Exception as err:
         logging.warning('Failed to obtain address: %r', err)
         return NO_VALUE_MSG, NO_VALUE_MSG
@@ -228,7 +231,7 @@ class AbstractConfigGenerator(abc.ABC):
 class SampleConfigGenerator(AbstractConfigGenerator):
     """Object representing the generated sample Patroni config.
 
-    Sane defults are used based on the gathered PG version.
+    Sane defaults are used based on the gathered PG version.
     """
 
     @property
@@ -313,7 +316,7 @@ class RunningClusterConfigGenerator(AbstractConfigGenerator):
 
     @property
     def _required_pg_params(self) -> List[str]:
-        """PG configuration prameters that have to be always present in the generated config.
+        """PG configuration parameters that have to be always present in the generated config.
 
         :returns: list of the parameter names.
         """
@@ -329,7 +332,7 @@ class RunningClusterConfigGenerator(AbstractConfigGenerator):
             :exc:`~patroni.exceptions.PatroniException`: if:
 
                 * pid could not be obtained from the ``postmaster.pid`` file; or
-                * :exc:`OSError` occured during ``postmaster.pid`` file handling; or
+                * :exc:`OSError` occurred during ``postmaster.pid`` file handling; or
                 * the obtained postmaster pid doesn't exist.
         """
         postmaster_pid = None
@@ -352,7 +355,7 @@ class RunningClusterConfigGenerator(AbstractConfigGenerator):
         """Get cursor for the PG connection established based on the stored information.
 
         :raises:
-            :exc:`~patroni.exceptions.PatroniException`: if :exc:`psycopg.Error` occured.
+            :exc:`~patroni.exceptions.PatroniException`: if :exc:`psycopg.Error` occurred.
         """
         try:
             conn = psycopg.connect(dsn=self.dsn,
@@ -438,7 +441,7 @@ class RunningClusterConfigGenerator(AbstractConfigGenerator):
             are located outside of ``PGDATA`` and Patroni doesn't have write permissions for them.
 
         :raises:
-            :exc:`~patroni.exceptions.PatroniException`: if :exc:`OSError` occured during the conf files handling.
+            :exc:`~patroni.exceptions.PatroniException`: if :exc:`OSError` occurred during the conf files handling.
         """
         default_hba_path = os.path.join(self.config['postgresql']['data_dir'], 'pg_hba.conf')
         if self.config['postgresql']['parameters']['hba_file'] == default_hba_path:
