@@ -8,7 +8,7 @@ from typing import Collection, List, NamedTuple, Optional, TYPE_CHECKING
 from .. import global_config
 from ..collections import CaseInsensitiveDict, CaseInsensitiveSet
 from ..dcs import Cluster
-from ..psycopg import quote_ident as _quote_ident
+from ..psycopg import quote_ident
 
 if TYPE_CHECKING:  # pragma: no cover
     from . import Postgresql
@@ -31,9 +31,14 @@ SYNC_REP_PARSER_RE = re.compile(r"""
         """, re.X)
 
 
-def quote_ident(value: str) -> str:
-    """Very simplified version of `psycopg` :func:`quote_ident` function."""
-    return value if SYNC_STANDBY_NAME_RE.match(value) else _quote_ident(value)
+def quote_standby_name(value: str) -> str:
+    """Quote provided *value* if it is nenecessary.
+
+    :param value: name of a synchronous standby.
+
+    :returns: a quoted value if it is required or the original one.
+    """
+    return value if SYNC_STANDBY_NAME_RE.match(value) and value.lower() not in ('first', 'any') else quote_ident(value)
 
 
 class _SSN(NamedTuple):
@@ -87,27 +92,27 @@ def parse_sync_standby_names(value: str) -> _SSN:
     >>> parse_sync_standby_names('1')  # doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
-    ValueError: Unparseable synchronous_standby_names value
+    ValueError: Unparsable synchronous_standby_names value
 
     >>> parse_sync_standby_names('a,')  # doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
-    ValueError: Unparseable synchronous_standby_names value
+    ValueError: Unparsable synchronous_standby_names value
 
     >>> parse_sync_standby_names('ANY 4("a" b,"c c")')  # doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
-    ValueError: Unparseable synchronous_standby_names value
+    ValueError: Unparsable synchronous_standby_names value
 
     >>> parse_sync_standby_names('FIRST 4("a",)')  # doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
-    ValueError: Unparseable synchronous_standby_names value
+    ValueError: Unparsable synchronous_standby_names value
 
     >>> parse_sync_standby_names('2 (,)')  # doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
-    ValueError: Unparseable synchronous_standby_names value
+    ValueError: Unparsable synchronous_standby_names value
     """
     tokens = [(m.lastgroup, m.group(0), m.start())
               for m in SYNC_REP_PARSER_RE.finditer(value)
@@ -137,10 +142,10 @@ def parse_sync_standby_names(value: str) -> _SSN:
     for i, (a_type, a_value, a_pos) in enumerate(synclist):
         if i % 2 == 1:  # odd elements are supposed to be commas
             if len(synclist) == i + 1:  # except the last token
-                raise ValueError("Unparseable synchronous_standby_names value %r: Unexpected token %s %r at %d" %
+                raise ValueError("Unparsable synchronous_standby_names value %r: Unexpected token %s %r at %d" %
                                  (value, a_type, a_value, a_pos))
             if a_type != 'comma':
-                raise ValueError("Unparseable synchronous_standby_names value %r: ""Got token %s %r while"
+                raise ValueError("Unparsable synchronous_standby_names value %r: ""Got token %s %r while"
                                  " expecting comma at %d" % (value, a_type, a_value, a_pos))
         elif a_type in {'ident', 'first', 'any'}:
             members.add(a_value)
@@ -150,7 +155,7 @@ def parse_sync_standby_names(value: str) -> _SSN:
         elif a_type == 'dquot':
             members.add(a_value[1:-1].replace('""', '"'))
         else:
-            raise ValueError("Unparseable synchronous_standby_names value %r: Unexpected token %s %r at %d" %
+            raise ValueError("Unparsable synchronous_standby_names value %r: Unexpected token %s %r at %d" %
                              (value, a_type, a_value, a_pos))
     return _SSN(sync_type, has_star, num, members)
 
@@ -198,7 +203,7 @@ class _ReplicaList(List[_Replica]):
 
     Values are reverse ordered by ``_Replica.sync_state`` and ``_Replica.lsn``.
     That is, first there will be replicas that have ``sync_state`` == ``sync``, even if they are not
-    the most up-to-date in term of write/flush/replay LSN. It helps to keep the result of chosing new
+    the most up-to-date in term of write/flush/replay LSN. It helps to keep the result of choosing new
     synchronous nodes consistent in case if a synchronous standby member is slowed down OR async node
     is receiving changes faster than the sync member. Such cases would trigger sync standby member
     swapping, but only if lag on this member is exceeding a threshold (``maximum_lag_on_syncnode``).
@@ -391,7 +396,7 @@ END;$$""")
         if has_asterisk:
             sync = ['*']
         else:
-            sync = [quote_ident(x) for x in sorted(sync)]
+            sync = [quote_standby_name(x) for x in sorted(sync)]
 
         if self._postgresql.supports_multiple_sync and len(sync) > 1:
             if num is None:
@@ -400,7 +405,7 @@ END;$$""")
         else:
             sync_param = next(iter(sync), None)
 
-        if global_config.is_quorum_commit_mode and sync or self._postgresql.supports_multiple_sync and len(sync) > 1:
+        if self._postgresql.supports_multiple_sync and (global_config.is_quorum_commit_mode and sync or len(sync) > 1):
             prefix = 'ANY ' if global_config.is_quorum_commit_mode and self._postgresql.supports_quorum_commit else ''
             sync_param = f'{prefix}{num} ({sync_param})'
 
@@ -408,7 +413,7 @@ END;$$""")
                 and self._postgresql.state == 'running' and self._postgresql.is_primary()) or has_asterisk:
             return
 
-        time.sleep(0.1)  # Usualy it takes 1ms to reload postgresql.conf, but we will give it 100ms
+        time.sleep(0.1)  # Usually it takes 1ms to reload postgresql.conf, but we will give it 100ms
 
         # Reset internal cache to query fresh values
         self._postgresql.reset_cluster_info_state(None)
